"""Download ERA-5 data from the Copernicus Climate Data Store."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

# builtins
import os
import pathlib
from joblib import Parallel, delayed

# externals
import cdsapi
import numpy as np

# ERA5 predictor variables on pressure levels
ERA5_P_VARIABLES = ['geopotential', 'temperature', 'u_component_of_wind',
                    'v_component_of_wind', 'specific_humidity']

# path to store downloaded ERA-5 data
ERA5_PATH = pathlib.Path('/mnt/CEPH_PROJECTS/FACT_CLIMAX/REANALYSIS/')

# user: Daniel or Alice
# @Alice: comment Daniel, activate Alice
user = 'Daniel'
# user = 'Alice'

# pressure levels
pressure_levels = ['1000', '850', '700', '500', '250']

# time period
years = [str(y) for y in np.arange(1981, 2011)]
month = [str(m) for m in np.arange(1, 13)]
days = [str(d) for d in np.arange(1, 32)]
time = ["{:02d}:00".format(t) for t in np.arange(0, 24)]

# area of interest (Alps): North, West, South, East
area = [52, 2, 40, 20]

# ERA5 download configuration dictionary
CONFIG = {
    'product_type': 'reanalysis',
    'month': month,
    'day': days,
    'time': time,
    'format': 'netcdf',
    'area': area
}

# whether to overwrite existing files
OVERWRITE = False

# whether to skip variables on single levels
SKIP_SINGLE_LEVELS = True

# variables depending on user
if user == 'Daniel':
    ERA5_VARIABLES = ['geopotential', 'temperature', 'u_component_of_wind',
                      'mean_sea_level_pressure', 'orography', '2m_temperature']

if user == 'Alice':
    ERA5_VARIABLES = ['v_component_of_wind', 'specific_humidity',
                      'surface_pressure', '2m_dewpoint_temperature',
                      'total_precipitation']


if __name__ == '__main__':

    # initialize client
    c = cdsapi.Client()

    # download data for the different variables
    for var in ERA5_VARIABLES:
        # create output directory
        output = ERA5_PATH.joinpath('Downloads', var)
        if not output.exists():
            output.mkdir(parents=True, exist_ok=True)

        # create output files
        files = [output.joinpath('_'.join(['ERA5', var, year]) + '.nc') for
                 year in years]

        # check whether to download variable on pressure levels or single level
        if var in ERA5_P_VARIABLES:
            # download configuration: ERA5 variable on pressure levels
            product = 'reanalysis-era5-pressure-levels'
            CONFIG['pressure_level'] = pressure_levels  # pressure levels
        else:
            # whether to skip single-level variables
            if SKIP_SINGLE_LEVELS:
                continue

            # "orography" variable was recently renamed to "geopotential": this
            # results in conflicting file names
            var = 'geopotential' if var == 'orography' else var

            # download configuration: ERA5 variable on single level
            product = 'reanalysis-era5-single-levels'
            if 'pressure_level' in CONFIG:
                _ = CONFIG.pop('pressure_level') # remove pressure levels

        # split the download to the different years: CDS API cannot handle
        # requests over the whole time period
        Parallel(n_jobs=min(len(years), os.cpu_count()), verbose=51)(
            delayed(c.retrieve)(
                product, {**CONFIG, **{'variable': var, 'year': year}}, file)
            for file, year in zip(files, years) if (not file.exists() or
            OVERWRITE))
